
# <b>Datenbank</b> regelt die Zugriffe auf die Datei "dateis" der Dateien sowie
# die Aktionen und sonstigen Informationen, die im Zusammenhang damit stehen 
#
# auftretende Fehler werden dabei als Programmierfehler betrachtet: keine Ausgabe ber @notice

class Datenbank
    
  attr_accessor :data, :treelist, :view_name, :treelist_header, :conn, :notice, :info
  cattr_reader :modul
  
  # <b>initialize</b> liest entweder die Datenstze der "dateis"-Datei ein oder
  # die Dateien der angebundenen Datenbank von Parameter "conn" ein (nur fr MySQL)
  # Soll eine Aktualisierung durchgefhrt werden, wird berprft, ob fr jede
  # der vorgefundenen Dateien bereits ein Eintrag in der "Datei der Dateien" (dateis) existiert.
  # * Wenn nicht, wird ein Datensatz angelegt.
  # * Wenn ja, wird der Datensatz ggf. aktualisiert
  # * Liegt sogar mehr als nur ein Datensatz vor, wird eine Fehlermeldung ausgegeben.
  
  def initialize(*parm)    # aufgerufen von new       
    @@dataview  = true  if !defined?(@@dataview)        # alle Verbindungen anzeigen  
    @@data      = {}    if !defined?(@@data) 
    @@prv       = {}    if !defined?(@@prv)
  
    if parm[0] == "modul"
      @@modul = sql('DISTINCT modul','') unless @@modul
    elsif parm[0] == "prv" && @@data.key?(@@prv)          # Bearbeitung fand bereits statt
      treelist_head
      @view_name =  @@prv.match('Navi') ? @@prv : "NaviSeite-1"    
    
    elsif parm[0] == "noDB" || parm[0] == "prv"
      @@prv = @view_name  = "NaviSeite-0"
      datei_to_tree                       # Einlesen der datei-Datenstze
    
    else      
      @view_name  = "NaviSeite-1"  
      check_conn(parm[0])                 # @conn als Voraussetzung fr das Einlesen der physikalischen DB  
      @@prv       = @conn.to_s
      update      = parm[1]            
      
      unless @notice  
        # die Angaben sollen aktualisiert werden im Falle von "update" sowie bei einer unbekannten Verbindung
        if !@@data.key?(@conn.to_s) || update       
          files                           # @@data
        end    
        view                              # @data     
        if update
          do_update
        end      
      end 

    end   # parm[0]

  end   #initialize 
  
  # <b>change_view</b> zeigt entweder alle Dateien an
  # oder reduziert sie auf die aktuelle Verbindung @conn
       
  def change_view
    @@dataview = !@@dataview
    view
  end  
  
 ######################################################################################################   
 
  private  
   
  # <b>datei_to_tree</b> liest die Datenstze aus der Datei ein und
  # bereitet die eingelesene Datenliste fr die Treelist-Anzeige auf
  
  def datei_to_tree
    raise "Fehler in #{File.basename(__FILE__)}, #{__LINE__}: \"#{Dateis.connection.adapter_name}\" beherrscht kein \"distinct\"" unless Dateis.connection.supports_count_distinct?
    # Abarbeiten der hinterlegten Adapter, Datenbanken und Module
    @treelist = do_group(1)
    @@data[@view_name]  = @treelist
    treelist_head
   end
  
  # <b>check_conn</b> prft die angeforderte Connection, ob sie aktiv ist
  # oder ob die Datenbank untersttzt wird. Wird keine Connection angegeben,
  # wird als Defaultwert "Datei" vorgegeben
 
  def check_conn(conn)
  
    conn  = conn || "Dateis" 
     
    if eval(conn).__id__       # existiert die Verbindung?
      @conn = ObjectSpace._id2ref(eval(conn).__id__) 
    else    
      @notice = "Die Verbindung &quot;#{conn}&quot; konnte nicht gefunden werden"
    end
    
    if @conn.connection.adapter_name != "MySQL" then
      @notice = " Datenbank &quot#{@conn.connection.adapter_name}&quot wird nicht unterst&uuml;tzt (nur MySQL)"
    elsif !@conn
      @notice = " Datenbank &quot#{@conn.to_s}&quot ist noch nicht angebunden"
    end
    
    @info = "error" if @notice

  end
   
  # <b>files</b> liest die Dateien der angebundenen Datenbank ein
  
  def files
     
      files             = {}
            
      tables            = @conn.connection.tables          # tables : definiert fr MySQL, PostgreSQL
      tables.each do |table|
        keys            = primary_key(table)
        file_full       = @conn.connection.columns(table)  # columns: definiert fr MySQL, PostgreSQL
        
        # Aufbereitung der Daten        
        file, type      = [], []
        file_full.each { |column|
          file << column.name
          type << column.type}
                
        files[table]    = keys, file, type, file_full
        
      end   #table
     
      @@data[@conn.to_s] = files      
      
  end  
  
  # <b>view</b> bereitet die Datenliste auf  
  # dabei wird die Liste alle bisher eingelesenen Daten auf die anzuzeigenden reduziert
  # (im Falle von @@dataview = false)
  # und fr diese Instanz ber <i>@data</i> zur Verfgung gestellt
  
  def view  
      tablefields, tablefieldsall       = {}, {}
     
      @@data.each do |conn, data|
            
        if conn.match('Navi')
          next                  # diese Routine ist nur fr die Bearbeitung der physikalischen Datei-Daten gltig
        elsif !@@dataview && @conn.to_s != conn
          next
        end

      data = data.sort
      
        # Schlsselfelder und Dateifelder befinden sich in den Arrays 0,2 der Datei-Arrays
        data.each do |table, fields|
          tablefields[table]    = fields[0,2]
          
          # fr Treelist: Feldnamen plus Feldinfo, aufbereitet fr treelist_helper
          sort                  = tree_fields(fields)               
          action                = ["edit_called_by_tree", "dateis"]  # Verarbeitung auf Datei-Ebene
          tablefieldsall[table] = [table, sort, [], action, [table, conn, @conn.connection.adapter_name] ]      # Treelist-Info-Array
                      
          end   # data
          
      
        # der Connection-Name wird in @data fr die Oberflche (application.rb) mitgegeben
        @data             = {}    if !defined?(@data)
        @data[conn] = tablefields
        
        # aufbereitete Form
        @treelist         = {}    if !defined?(@treelist)  
        action            = []                     # bisher keine Aktionen auf Ebene der Adapter/Datenbanken
        @treelist[conn]   = [@conn.connection.current_database, tablefieldsall, [], action, [@conn.connection.current_database]]  # nur MySQL
        
      end   # @@data
      
        treelist_head
  end
  
  # <b>update</b> berprft, ob fr jede der vorgefundenen Dateien 
  # bereits ein Eintrag in der "Datei der Dateien" (dateis) existiert.
  # * Wenn nicht, wird ein Datensatz angelegt.
  # * Wenn ja, wird der Datensatz ggf. aktualisiert
  # * Liegt sogar mehr als nur ein Datensatz vor, wird eine Fehlermeldung ausgegeben.
  
  def do_update

    @notice = "Datei der Dateien wurde aktualisiert<br>"
    @info   = "hinweis"    
    datei_conn  = eval("Dateis")                          # Modell der Datei der Dateien  
    
    @data[@conn.to_s].each do |table, fields|
    
      # fr die gesuchte Datei "table" werden alle Eintrge in der Datei der Dateien bestimmt
      # falls mehr als einer vorliegt, ist das ein Fehler, wenn weniger vorliegt, muss ein Satz erstellt werden 
      files       = datei_conn.find_all_by_name(table)

      if files.length > 1
       @notice = @notice, " vorgefundene Dateien f&uuml;r den Schl&uuml;ssel &quot;#{table}&quot; = #{files.length} <i>(bitte selbst pr&uuml;fen und &auml;ndern!)</i> </br>"      
       @info   = "error"
      else
        if  files.length  == 0
          datei           = datei_conn.new
        else
          datei           = files[0]
        end
      
        attributes                  = datei.attributes
        attributes["name"]          = table
        attributes["beschreibung"]  = table if attributes["beschreibung"].empty?
        attributes["adapter"]       = @conn.connection.adapter_name         # abstract_adapter.rb: adapter_name ist gltiges Attribut aller Adapter   
        attributes["datenbank"]     = @conn.connection.current_database     # nur MySQL
        attributes["key"]           = fields[0].join(", ")

        attributes["fields"]        = Marshal_out(fields, @conn.connection.columns(table))
      
        unless attributes == datei.attributes
          datei.attributes   = attributes
          datei.save
        end
        
      end   #files.length
      
    end   #table
  end
  
  ######## 2. Level    
   
  # fr Treelist: Feldnamen plus Feldinfo, aufbereitet fr treelist_helper 
  # * sort       : Sortiermerkmal
  # * Array      : standardisierte Form (Info-Array)
  #   1. Array[0]: key           - anzuzeigendes Element
  #   2. Array[1]: Hash          - enthlt den nchsten Zweig, falls vorhanden
  #   3. Array[2]: Array         - URL/Name des Icons, Alt/Tooltip zum Image
  #   4. Array[3]: Array         - Aktion und Controller
  #   5. Array[4]: Array         - enthlt weitere Informationen fr Tooltips, Rechte Maus
  
  def tree_fields(fields,*parm)  
    sort                  = {}        # Hash der anzuzeigenden Felder
    icon, detail          = []        # Arrays der weiteren Informationen
        
    (0...fields[1].length).each do |i|
      type                = ""
      if defined?(fields[2][i])
        type = "Feldtyp = #{fields[2][i]}"
      else
        type = "Feldtyp = #{parm[0][i]}"
      end
      detail                  = fields[1][i], "Feldnummer = #{i+1}", type
  
      ic                      = fields[0].include?(fields[1][i]) ? "tree/key" : ""
      icon                    = [ic, ic.empty? ?   "" : "Schluesselfeld" ]
      
      sort[i.to_s] =                                # Sortiermerkmal
         fields[1][i],                              # key
         {},                                        # leeres Hash - kein weiterer Zweig 
         icon,                                      # Array: Icon-URL, Alt/Tooltip
         [],                                        # leeres Array - keine Aktionen auf Feldebene
         detail                                     # Array: Aktionen, Tooltips, Rechte Maus
      end   # i
    sort
  end  
  
  # da connection.indexes gerade den Primary Key berspringt
  # und da ich keine Rails-Funktion finden kann, muss der Primrschlssel selbst bestimmt werden
  
  def primary_key(table)
    sql = "SHOW KEYS FROM #{table} WHERE key_name = 'PRIMARY'"
    keys = []
    @conn.connection.execute(sql).each { |row| keys << row[4]} # execute: definiert fr MySQL, PostgreSQL
    keys                                                       # row[4] ist Feldname
  end
 
 # <b>Marshal_out</b> bereitet die Daten fr das Dateifeld "fields" so auf,
 # dass beim Zurckspeichern die Prozedur "tree_fields" aufgerufen werden kann,
 # um die Feldinformationen identisch aufbereitet zu erhalten.
 # Dies erfolgt, da die aufbereiteten Informationen selbst nicht mehr "marshalbar" sind (zu komplex).
  
  def Marshal_out(sort, conn)
    type = []
    conn.each {|element| type << element.type}
    Marshal.dump([sort, type])
  end
  
  # <b>do_group</b> arbeitet die hinterlegten Adapter, Datenbanken und Module ab 
  
  def do_group(level, *parm)
    adapter   = parm[0] || ""
    datenbank = parm[1] || ""
    modul     = parm[2] || ""
    
    items   = {}
    
    # passende Datenstze finden
    group = find(level, adapter, datenbank, modul)
    
    # Aufbereiten der einzelnen Hierarchien
    mbr, todo, icon, action  = get_level(level)    

    group.each do |member|
      items[member[mbr[0]]] = [member[mbr[1]], eval(todo.to_s), icon, action, [member[mbr[1]]]]       # Array: key, Hash, Icon-Array, Action-Array, Info-Array
      end
    items
  end
  
  # <b>find</b> ruft den SQL-Befehl auf, da der Rails-Find
  # die Anordnung der Felder zerhagelt
   
  def find(level, ad, db, md)
    values  = get_level(level) 
    where   = val_to_where(values[5], ad, db, md)
    sql(values[4],where)                                         
  end
  
  # <b>sql</b> fhrt den SQL-Befehl aus und bergibt das Ergebnis
  
  def sql(select, where)
    sql = "SELECT #{select} FROM dateis #{where}"
    group = []
    Dateis.connection.execute(sql).each { |row| group << row}   # execute/distinct: definiert fr MySQL, PostgreSQL
    group  
  end
  
  # <b>val_to_where</b> erledigt die Aufbereitung der Parameter
  
  def val_to_where(where, ad, db, md)
    where   = where.gsub('\'ad\'', "\'#{ad}\'")
    where   = where.gsub('\'db\'', "\'#{db}\'") 
    where   = where.gsub('\'md\'', "\'#{md}\'") 
  end
  
  # <b>Marshal_in</b> bereitet die Feldinformationen auf
  
  def Marshal_in(stored)
    begin
      fields_and_types  = Marshal.load(stored)
      fields            = fields_and_types[0]  
      types             = fields_and_types[1] 
      
      tree_fields(fields,types)
    rescue              # leere Daten sollte heien, dass der Datei-Datensatz noch nicht mit den Informationen aus der physikalischen DB versorgt wurde
      {}                # leeres Hash - keine weitere Treelist-Verzweigung
    end    
  end

  ######## Vorgaben
     
  # <b>get_level</b> bestimmt die Hierarchie-Ebenen
  
  def get_level(level)
    case level
    when 1
      mbr    = [0,0]
      todo   = "do_group(2, member[0])"
      icon   = []
      action = []                               # bisher keine Aktionen auf Ebene der Adapter
      member = "DISTINCT adapter"
      where  = ""
    when 2
      mbr    = [1,1]
      todo   = "do_group(3, adapter, member[1])"
      icon   = []
      action = []                                 # bisher keine Aktionen auf Ebene der Datenbanken    
      member = "DISTINCT adapter, datenbank"
      where  = "where adapter = \'ad\'" 
    when 3
      mbr    = [2,2]
      todo   = "do_group(4, adapter, datenbank, member[2])"
      icon   = []      
      action = ["list_modul", "dateis"]      
      member = "DISTINCT adapter, datenbank, modul"
      where  = "where adapter = \'ad\' and datenbank = \'db\'"      
    else
      mbr    = [0,1]
      todo   = "Marshal_in(member[9])"
      icon   = []      
      action = ["edit_called_by_tree", "dateis"]  # Verarbeitung auf Datei-Ebene
      member = "*"
      where  = "where adapter = \'ad\' and datenbank = \'db\' and modul = \'md\'"                      
    end
    return mbr, todo, icon, action, member, where  
  end
  
  # <b>treelist_head</b> bereitet die Kopfinformationen der Treelist auf
  
  def treelist_head
    @treelist_header    = []
    if @view_name == "NaviSeite-0" 
      @treelist_header[0] = "Dateiverzeichnis"                      # Root-Name
    else
      @treelist_header[0] = "Datenbankverzeichnis"                  # Root-Name    
    end
    @treelist_header[1] = ["tree/root", "Datenbanken und Dateien"]  # icon, alt/tooltip 
    @treelist_header[2] = []                                        # keine Verarbeitung auf Root-Ebene
  end

end     # class
