
# <b>Treelist</b> erstellt aus standardisierten Vorgaben eine einfache HTML-Baumstruktur
#
# auftretende Fehler werden dabei als Programmierfehler betrachtet: keine Ausgabe ber @notice

class Treelist

  attr_accessor :tree_html

 # <b>initialize</b> bereitet die bergebene Treelist "@treelist" auf
 # @treelist wird dabei als Hash der Form [key => Array] erwartet.
 # * sort       : Sortiermerkmal
 # * Array      : standardisierte Form (Info-Array)
 #   1. Array[0]: key           - anzuzeigendes Element
 #   2. Array[1]: Hash          - enthlt den nchsten Zweig, falls vorhanden
 #   3. Array[2]: Array         - URL/Name des Icons, Alt/Tooltip zum Image
 #   4. Array[3]: Array         - Aktion und Controller
 #   5. Array[4]: Array         - enthlt weitere Informationen fr Tooltips, Rechte Maus
 #
 # @treelist wird dann in eine Klassenvariable @@tree berfhrt, die die Trees zeilenweise auflistet
 # aus @@tree wird dann die aktuelle HTML-Ansicht erzeugt

  def initialize(*parm)   
    
    @partial              = parm[0]
    @treelist             = parm[1]
    @treelist_header      = parm[2]
    @default              = parm[3]

    level, @html          = 64, []        
    tree(@treelist, level)    
    do_html('')
          
  end
  
  # <b>html</b> bereitet die auszugebenden Zeilen auf  
  
  def do_html(given)
      collapse  = defined?(@tree_html)  ?  @tree_html[0][2] : @default
      
      @tree_html, line, @id   = [], [], {}
      @frozen                 = ''
      collapse  = check_hide(given == @partial, collapse, "A")    
      
      # first line      
      line                    << "<ul class=\"root\" id = \"#{@partial}\">" 
      line                    << @partial
      line                    << collapse                    # 1,-1
      line                    << @treelist_header[1]            # Tree-Icon-Alt/Tooltip            
      line                    << @treelist_header[0]            # Tree-Anzeige 
      line                    << @treelist_header[2]            # Tree-Aktion
      @tree_html              << line
 
      do_line(given)  if @frozen.empty?
        
      # last line      
      line                    = []
      line                    << "</ul>"
      @tree_html              << line
                    
  end

 ######################################################################################################   
   
  private   
  
  # <b>tree</b> bringt die bergebene Treelist in direkt aufeinander folgende Zeilen
  
  def tree(treelist, level)
      level     = level + 1      
      
      treelist  = treelist.sort
      
      treelist.each do |parent, child| 
          raise "Fehler in #{File.basename(__FILE__)}, #{__LINE__}: \"#{child.class.name}\"" unless child.class.name == 'Array' && child[1].class.name == 'Hash'
          raise "Fehler in #{File.basename(__FILE__)}, #{__LINE__}: \"#{parent}\"" if parent.nil? || parent.empty?          
          raise "Fehler in #{File.basename(__FILE__)}, #{__LINE__}: \"#{child[0]}\"" if child[0].nil? || child[0].empty?          
                       
          child[1].empty? ? collapse = "0" : collapse = @default
          
          html      = collapse,
                      level.chr,
                      child[2],                 # Array: URL/Name des Icons, Alt/Tooltip zum Image                  
                      child[0],                 # String: anzuzeigender Key
                      child[3],                 # Array: Aktion und Controller
                      child[4],                 # Array: sonstige Info                        
                      child[1].empty?
                
          @html << html   
       
          tree(child[1], level) if !child[1].empty?                     
          
        end   #  treelist       

  end
 
  def do_line(given)
  
      (0...@html.length).each do |i|
  
        content               = @html[i]
        level                 = content[1]
         
        # Hierarchiewechsel?
        i < @html.length-1    ?   top = @html[i+1][1] : top = "A"
        
        # Zweig soll verborgen werden
        unless @frozen.empty?
          if (@frozen < @html[i][1])
            hierarchy(@frozen, top)
            next
          else
            @frozen = ''
            last_line_group 
          end
        end
         
        line                  = []
 
        id_html               = @partial + "-" + i.to_s
        content[0] = check_hide(given == id_html, content[0], level)
        
        if content.last == false      # parent      
          line                << "<li class=\"parent\" id = \"#{id_html}\"><ul class=\"parent\">"
        else
          if top < level
              line            << "<li class=\"childlast\" id = \"#{id_html}\">"            
            else
              line            << "<li class=\"child\" id = \"#{id_html}\">"              
          end
        end  
        
        line               << id_html               # id
        line               << content[0]            # +/-
        if (content[2].empty? || content[2][0].empty?) && content.last == true
          line            << ["tree/ohne" , ""]     # icon-id, alt/tooltip
        else
          line            << content[2]             # icon-id, alt/tooltip
        end         
        
        line              << content[3]             # Key
        line              << content[4]             # Aktion und Controller        
        line              << content[5][0]          # Tooltip
        
        if content.last == true                     # child   
          line            << "</li>"  
        end
        
        @tree_html        << line
      
        # Hierarchie-Ebene abgearbeitet?
        hierarchy(level, top)

      end     # each      
   end      # do_line
   
   # <b>last_line_group</b> schreibt die abschlieende Zeile eines Zweiges
   
   def last_line_group
      line           = []
      line           << "</ul></li>"
      @tree_html      << line  
   end
   
   # <b>check_hide</b> setzt den Level, ab dem nichts mehr angezeigt werden soll
   
   def check_hide(todo, collapse, level)
      if todo
        collapse  = (collapse.to_i * (-1)).to_s
      end
      if collapse == '1' && @frozen.empty?
        @frozen     = level       
      end
      collapse     
   end
   
   # <b>hierarchy</b> setzt die HTML-Abschluss-Tags fr die Listengruppen
   
   def hierarchy(level,top) 
      while level > top
        last_line_group 
        top = top.succ
      end
   end
   
end     # class
